;+
; NAME:
;	pntfit
; PURPOSE: (one line)
;	Fit model to telescope pointing data.
; DESCRIPTION:
; CATEGORY:
; CALLING SEQUENCE:
;	pntfit,raobs,decobs,ratrue,dectrue,sidtime,coefs
; INPUTS:
;	raobs   - Observed right ascension in degrees.
;	decobs  - Observed declination in degrees.
;	ratrue  - Catalog right ascension in degrees.
;	dectrue - Catalog declination in degrees.
;	sidtime - Local sidereal time at time of observations.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD PARAMETERS:
;	TITLE   - Label to put on plot.
; OUTPUTS:
;	coefs   - Pointing coefficients.
;	rafix   - Observed right ascension corrected by model (degrees).
;	decfix  - Observed declination corrected by model (degrees).
;	delr    - Correction applied to right ascension in arc sec.
;	deld    - Correction applied to declination in arc sec.
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;	2/11/93 - Written by Marc W. Buie, Lowell Observatory.
;-
pro pntfit,raobs,decobs,ratrue,dectrue,sidtime,coefs,rafix,decfix,delr,deld, $
            TITLE=title

haobs = sidtime-raobs
ha    = sidtime-ratrue

lat =  35.09683102d0  ; Larry's values
lon = 111.5359167d0

;lat=35.0967189d0    ; GPS value for 42"
;lon=111.5365053d0

lat_r     = lat     / 180.0d0 * !dpi
lon_r     = lon     / 180.0d0 * !dpi
haobs_r   = haobs   / 180.0d0 * !dpi
ha_r      = ha      / 180.0d0 * !dpi
decobs_r  = decobs  / 180.0d0 * !dpi
ratrue_r  = ratrue  / 180.0d0 * !dpi
dectrue_r = dectrue / 180.0d0 * !dpi

xx = cos(ha_r)
yy = sin(ha_r)
zz = sin(lat_r) * cos(dectrue_r) - cos(lat_r) * sin(dectrue_r) * xx
xind = [[xx],[yy],[zz]]
xind = transpose(xind)
yind = (dectrue - decobs) * 3600.0d0
wt = yind*0+1.0d0
regression,xind,yind,wt,const,coeff,resid,yfit, $
   sigma,ftest,r,rmul,chisqr,/noprint
print,xind[*,0]
print,yind[0]

a0 =  const
a1 = -coeff[0]
a2 = -coeff[1]
a3 = -coeff[2]

print,'Declination fit: Chisqr',chisqr,'   R = ',rmul
print,'  a0 = ',a0
print,'  a1 = ',a1,' +/- ',sigma[0],'   r = ',r[0]
print,'  a2 = ',a2,' +/- ',sigma[1],'   r = ',r[1]
print,'  a3 = ',a3,' +/- ',sigma[2],'   r = ',r[2]
print,' Scatter ',mean(abs(resid)),' arc-sec',format='(a,f6.2,a)'
print,' RMS     ',sqrt(total(resid^2)/(n_elements(resid)-1))

ss = 1.0/cos(dectrue_r)
tt = tan(dectrue_r)
qq = sin(lat_r) * tan(dectrue_r) + cos(lat_r) * cos(ha_r)
hh = -a1 * yy + a2 * xx + a3 * cos(lat_r) * ss * yy

xind2 = [[ss],[tt],[qq],[ha]]
xind2 = transpose(xind2)
yind2 = (ha - haobs) * 3600.0d0 - hh

regression,xind2,yind2,wt,const2,coeff2,resid2,yfit2, $
   sigma2,ftest2,r2,rmul2,chisqr2,/noprint

print,xind2[*,0]
print,yind2[0]

b0 =  const2
b1 =  coeff2[0]
b2 = -coeff2[1]
b3 =  coeff2[2]
b4 =  coeff2[3]

print,'Right Ascension fit: Chisqr',chisqr2,'   R = ',rmul2
print,'  b0 = ',b0
print,'  b1 = ',b1,' +/- ',sigma2[0],'   r = ',r2[0]
print,'  b2 = ',b2,' +/- ',sigma2[1],'   r = ',r2[1]
print,'  b3 = ',b3,' +/- ',sigma2[2],'   r = ',r2[2]
print,'  b4 = ',b4,' +/- ',sigma2[3],'   r = ',r2[3]
print,' Scatter ',mean(abs(resid2*cos(dectrue_r))),' arc-sec',format='(a,f6.2,a)'
print,' RMS     ',sqrt(total((resid2*cos(dectrue_r))^2)/(n_elements(resid2)-1))

deld = a0 - a1 * xx - a2 * yy - a3 * zz
delr = b0 + b1 * ss - b2 * tt + hh + b3 * qq + b4 * ha

rafix  = raobs  - delr/3600.0d0
decfix = decobs + deld/3600.0d0

; Reform coefficients to put in physical domain.
coefs    = dblarr(9)
coefs[0] = a0
coefs[1] = b0
coefs[2] = sqrt(a1^2 + a2^2)
coefs[3] = atan(a2,a1) * !radeg
coefs[4] = a3
coefs[5] = b1
coefs[6] = b2
coefs[7] = b3
coefs[8] = b4

if coefs[3] lt 0 then coefs[3] = coefs[3] + 360.0d0

;Print physical coefficients
print,coefs[0],'  Zero point in dec (")'
print,coefs[1],'  Zero point in ra (")'
print,coefs[2],'  Angle between true and instrumental poles (")'
print,coefs[3],'  Angle between line of pole and true meridian (deg)'
print,coefs[4],'  Telescope tube droop in HA and DEC (")'
print,coefs[5],'  Angle between optical and telescope tube axes (")'
print,coefs[6],'  Mechanical orthogonality of RA and DEC axes (")'
print,coefs[7],'  Dec axis flexure (")'
print,coefs[8],'  HA encoder scale error ("/degree)'

if !d.name eq 'SUN' or !d.name eq 'X' then begin
   device,window_state=win_state
   if win_state[1] eq 1 then wset,1 else window,1
endif

p_multi=!p.multi
!p.multi=[0,2,2]
plot,ha,yind,psym=4,ytit='Dec difference (")',xtit='E     Hour Angle (degrees)     W'
plot,ha,resid,psym=4,ytit='Dec residual (")',xtit='E     Hour Angle (degrees)     W'
plot,ha,yind2*cos(dectrue_r),psym=4,ytit='HA difference (")',xtit='E     Hour Angle (degrees)     W'
plot,ha,resid2*cos(dectrue_r),psym=4,ytit='HA residual (")',xtit='E     Hour Angle (degrees)     W'
if keyword_set(title) then xyouts,0.5,0.98,title,align=0.5,/normal
!p.multi=0

if !d.name eq 'SUN' or !d.name eq 'X' then wset,0

end
